
#import "SSHConnection.h"

#include "libssh2.h"
#include "libssh2_config.h"
#include "libssh2_sftp.h"
#include <sys/socket.h>
#include <arpa/inet.h>


#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>



unsigned long hostaddr;
int sock;
struct sockaddr_in soin;
LIBSSH2_SESSION *session;
LIBSSH2_CHANNEL *channel;
int rc;

static int waitsocket(int socket_fd, LIBSSH2_SESSION *session)
{
    struct timeval timeout;
    int rc;
    fd_set fd;
    fd_set *writefd = NULL;
    fd_set *readfd = NULL;
    int dir;
	
    timeout.tv_sec = 10;
    timeout.tv_usec = 3;
	
    FD_ZERO(&fd);
	
    FD_SET(socket_fd, &fd);
	
    /* now make sure we wait in the correct direction */
    dir = libssh2_session_block_directions(session);
	
    if(dir & LIBSSH2_SESSION_BLOCK_INBOUND)
        readfd = &fd;
	
    if(dir & LIBSSH2_SESSION_BLOCK_OUTBOUND)
        writefd = &fd;
	
    rc = select(socket_fd + 1, readfd, writefd, NULL, &timeout);
	
    return rc;
}

@implementation SSHConnection

-(int) connectToHost:(NSString *)host port:(int)port user:(NSString *)user password:(NSString *)password {
	const char* hostChar = [host cStringUsingEncoding:NSUTF8StringEncoding];
	const char* userChar = [user cStringUsingEncoding:NSUTF8StringEncoding];
	const char* passwordChar = [password cStringUsingEncoding:NSUTF8StringEncoding];
    
        
    
   // NSString *nameip = [[NSUserDefaults standardUserDefaults] stringForKey:@"mul2Value"];
    
        
        struct hostent *he;
        struct in_addr **addr_list;
        he = gethostbyname(hostChar);
        if(he!=nil){
        addr_list = (struct in_addr **)he->h_addr_list;
        for(int i = 0; addr_list[i] != NULL; i++) {
            hostChar=inet_ntoa(*addr_list[i]);
            
        }}
        


    
    hostaddr = inet_addr(hostChar);
    
    
    sock = socket(AF_INET, SOCK_STREAM, 0);
    soin.sin_family = AF_INET;
    soin.sin_port = htons(port);
    soin.sin_addr.s_addr = hostaddr;
    if (connect(sock, (struct sockaddr*)(&soin),sizeof(struct sockaddr_in)) != 0) {
        
        return -1;
    }
	
    
    NSLog(@"CREA LA SESION");
    /* Create a session instance */
    session = libssh2_session_init();

       
    if (!session)
        return -1;
	
    /* tell libssh2 we want it all done non-blocking */
    libssh2_session_set_blocking(session, 0);
	
    /* ... start it up. This will trade welcome banners, exchange keys,
     * and setup crypto, compression, and MAC layers
     */
    while ((rc = libssh2_session_startup(session, sock)) ==
           LIBSSH2_ERROR_EAGAIN);
    if (rc) {
        
    
        return -1;
    }

    if ( strlen(passwordChar) != 0 ) {
		/* We could authenticate via password */
        while ((rc = libssh2_userauth_password(session, userChar, passwordChar)) == LIBSSH2_ERROR_EAGAIN);
		if (rc) {
             NSLog(@"DEVUELVE 1!!!");
        
			return 1;
		}
	}
    NSLog(@"DEVUELVE 0!!!");
	return 0;
}

-(NSString *)executeCommand:(NSString *)command {
	const char* commandChar = [command cStringUsingEncoding:NSUTF8StringEncoding];

	NSString *result=@"";
	
     //Exec non-blocking on the remove host */
    while( (channel = libssh2_channel_open_session(session)) == NULL &&
		  libssh2_session_last_error(session,NULL,NULL,0) == LIBSSH2_ERROR_EAGAIN )
    {
        waitsocket(sock, session);
    }
    
      
    
    
    if( channel == NULL )
    {
        return @"error – channel not valid";
      //  exit( 1 );
        
    }
    while( (rc = libssh2_channel_exec(channel, commandChar)) == LIBSSH2_ERROR_EAGAIN )
    {
        waitsocket(sock, session);
    }
    if( rc != 0 )
    {
        return @"error – command didn’t work";
        //exit( 1 );
    }
    NSLog(@"%d",rc);
    bool entra=false;
    for( ;; )
    {
        /* loop until we block */
        int rc1;
        do
        {
            char buffer[0x20000];
            rc1 = libssh2_channel_read( channel, buffer, sizeof(buffer) );
            if( rc1 > 0 )
            {
				result = [NSString stringWithCString:buffer encoding:NSASCIIStringEncoding];
                entra=true;
            }
            NSLog(@"%d,",rc1);
        }
        while( rc1 > 0 );
		
        /* this is due to blocking that would occur otherwise so we loop on
		 this condition */
        if( rc1 == LIBSSH2_ERROR_EAGAIN )
        {
            waitsocket(sock, session);
        }
        else
            break;
    }
    while( (rc = libssh2_channel_close(channel)) == LIBSSH2_ERROR_EAGAIN )
        waitsocket(sock, session);
	
    libssh2_channel_free(channel);
    channel = NULL;
    NSString *myString =@"--------------------------" ;
    result= [result stringByAppendingString:myString];
    
	if(entra) return result;
    else return @"The input command didn't receive information.";
	
}


-(int) closeConnection {	
    libssh2_session_disconnect(session,"Normal Shutdown, Thank you for playing");
    libssh2_session_free(session);
	
    close(sock);


    //fprintf(stderr, "Connection closed\n");
    
	
	return 0;
}

@end
